const moment_timezone = require("moment-timezone");
const {BaseFilter,BaseQuery} = require("@cubejs-backend/schema-compiler");
const GRANULARITY_TO_INTERVAL = {
    day: (date) => `DATE_FORMAT(${date}, '%Y-%m-%dT00:00:00.000')`,
    week: (date) => `DATE_FORMAT(DATE_ADD('1900-01-01', INTERVAL TIMESTAMPDIFF(WEEK, '1900-01-01', ${date}) WEEK), '%Y-%m-%dT00:00:00.000')`,
    hour: (date) => `DATE_FORMAT(${date}, '%Y-%m-%dT%H:00:00.000')`,
    minute: (date) => `DATE_FORMAT(${date}, '%Y-%m-%dT%H:%i:00.000')`,
    second: (date) => `DATE_FORMAT(${date}, '%Y-%m-%dT%H:%i:%S.000')`,
    month: (date) => `DATE_FORMAT(${date}, '%Y-%m-01T00:00:00.000')`,
    quarter: (date) => `DATE_ADD('1900-01-01', INTERVAL TIMESTAMPDIFF(QUARTER, '1900-01-01', ${date}) QUARTER)`,
    year: (date) => `DATE_FORMAT(${date}, '%Y-01-01T00:00:00.000')`
};
class MysqlFilter extends BaseFilter {
    likeIgnoreCase(column, not, param) {
        return `${column}${not ? ' NOT' : ''} LIKE CONCAT('%', ${this.allocateParam(param)}, '%')`;
    }
}
class MysqlQuery extends BaseQuery {
    newFilter(filter) {
        return new MysqlFilter(this, filter);
    }
    convertTz(field) {
        return `CONVERT_TZ(${field}, @@session.time_zone, '${moment_timezone().tz(this.timezone).format('Z')}')`;
    }
    timeStampCast(value) {
        return `TIMESTAMP(convert_tz(${value}, '+00:00', @@session.time_zone))`;
    }
    timestampFormat() {
        return moment_timezone.HTML5_FMT.DATETIME_LOCAL_MS;
    }
    dateTimeCast(value) {
        return `TIMESTAMP(${value})`;
    }
    subtractInterval(date, interval) {
        return `DATE_SUB(${date}, INTERVAL ${interval})`;
    }
    addInterval(date, interval) {
        return `DATE_ADD(${date}, INTERVAL ${interval})`;
    }
    timeGroupedColumn(granularity, dimension) {
        return `CAST(${GRANULARITY_TO_INTERVAL[granularity](dimension)} AS DATETIME)`;
    }
    escapeColumnName(name) {
        return `\`${name}\``;
    }
    seriesSql(timeDimension) {
        const values = timeDimension.timeSeries().map(([from, to]) => `select '${from}' f, '${to}' t`).join(' UNION ALL ');
        return `SELECT TIMESTAMP(dates.f) date_from, TIMESTAMP(dates.t) date_to FROM (${values}) AS dates`;
    }
    concatStringsSql(strings) {
        return `CONCAT(${strings.join(', ')})`;
    }
    unixTimestampSql() {
        return 'UNIX_TIMESTAMP()';
    }
    wrapSegmentForDimensionSelect(sql) {
        return `IF(${sql}, 1, 0)`;
    }
    preAggregationTableName(cube, preAggregationName, skipSchema) {
        const name = super.preAggregationTableName(cube, preAggregationName, skipSchema);
        if (name.length > 64) {
            throw new UserError_1.UserError(`MySQL can not work with table names that longer than 64 symbols. Consider using the 'sqlAlias' attribute in your cube and in your pre-aggregation definition for ${name}.`);
        }
        return name;
    }
}
module.exports = MysqlQuery;
